import os
import sys
import getopt
import antelope.stock as stock
import antelope.elog as elog
import antelope.datascope as datascope


pfname = 'rtwebserver'
pfptr = None
sitename = 'site'

port = -1
daemonize = False
verbose = False
profile = False
diagnostics = False
trackingdbname = None
dbtrack = None
valid_ip_addresses = list()
reject_ip_addresses = list()


def usage():
    elog.notify("Usage: rtwebserver [-dv] [-p pfname] [-P port] [-s sitename] [pfplainref value "
                "[pfplainref value]...]\n")


def inform(message):

    if verbose:
        elog.notify("%s\n" % message)


def configure(args):

    try:
        opts, pargs = getopt.getopt(sys.argv[1:], 'dvp:P:s:')
    except getopt.GetoptError:
        usage()
        sys.exit(-1)

    if len(pargs) % 2 != 0:
        usage()
        sys.exit(-1)

    plainrefs = dict(list(zip(pargs[0::2], pargs[1::2])))

    elog.init(sys.argv)

    elog.notify("Starting rtwebserver at '%s' as pid %d" %
                (stock.epoch2str(stock.now(), "%T %D %Z"), os.getpid()))

    for option, value in opts:

        if option in ['-d']:
            globals()['diagnostics'] = True
            globals()['verbose'] = True

        if option in ['-p']:
            globals()['pfname'] = value

        if option in ['-v']:
            globals()['verbose'] = True

    pfname = globals()['pfname']

    try:
        pfptr = stock.pfread(pfname)
    except stock.PfReadError as e:
        elog.die("Failed to read parameter-file %s (%d)" % (e[0], e[1]))

    for key in list(plainrefs.keys()):

        pfptr[key] = plainrefs[key]

        if globals()['verbose'] is True:

            elog.notify("Inserted into parameter-file '%s' the key/value pair from the command-line:\n\n\t%s\t%s\n\n" %
                        (globals()['pfname'], key, plainrefs[key]))

    pfdict = pfptr.pf2dict()

    if 'profile' in pfdict:

        globals()['profile'] = stock.yesno(pfdict['profile'])

    if 'diagnostics' in pfdict:

        globals()['diagnostics'] = stock.yesno(pfdict['diagnostics'])

        if globals()['diagnostics'] is True: 

            globals()['verbose'] = True

    globals()['port'] = int(pfdict['port'])
    globals()['trackingdbname'] = pfdict['trackingdb']

    for option, value in opts:
        if option in ['-P']:
            globals()['port'] = int(value)

        if option in ['-s']:
            globals()['sitename'] = value

    try:
        globals()['sitedict'] = pfdict[globals()['sitename']]
    except:
        elog.complain("Could not find web-site array named '%s' in parameter-file '%s'. Exiting.\n" %
                      (globals()['sitename'], pfname))
        sys.exit(-1)

    try:
        globals()['valid_ip_addresses'] = pfdict['valid_ip_addresses']
    except:
        elog.complain("Could not find valid_ip_addresses table in parameter-file '%s'. Exiting.\n" % pfname)
        sys.exit(-1)

    if trackingdbname is not None and trackingdbname != '':

        if not os.path.exists(trackingdbname):
            datascope.dbcreate(trackingdbname, 'Rtwebtrack0.1')

        globals()['dbtrack'] = datascope.dbopen(trackingdbname, 'r+')

        if globals()['dbtrack'].database < 0:

            elog.complain("Failed to open tracking database '%s': disabling database tracking of web requests" %
                          trackingdbname)
            globals()['dbtrack'] = None

        elif verbose:
            elog.notify("Opened database '%s' to track web requests\n" % trackingdbname)
    else:
        globals()['dbtrack'] = None

    return


configure(sys.argv)
