#   Copyright (c) 2012 Boulder Real Time Technologies, Inc.           
#                                                                     
#   This software module is wholly owned by Boulder Real Time         
#   Technologies, Inc. Any use of this software module without        
#   express written permission from Boulder Real Time Technologies,   
#   Inc. is prohibited.                                               

import os
import re
import math
from PIL import Image, ImageDraw


import antelope.orb as orb
import antelope.Pkt as Pkt
import antelope.elog as elog
import antelope.stock as stock
import antelope.coords as coords
import antelope.datascope as datascope

from twisted.web import http

class MapManager(object):

    facilities = None
    mainmap = None
    stas = dict()
    db = None

    def __init__( self, pfdict ):

        self.sitedict = pfdict

        MapManager.db = datascope.dbopen( self.sitedict['site_database'] )
        MapManager.db = MapManager.db.lookup( table='site' )
        MapManager.db = MapManager.db.subset( "offdate == NULL" )

        for MapManager.db.record in range( MapManager.db.query( datascope.dbRECORD_COUNT ) ):

            (name, lat, lon) = MapManager.db.getv( "sta", "lat", "lon" )

            MapManager.stas[name] = dict()
            MapManager.stas[name]['lat'] = lat
            MapManager.stas[name]['lon'] = lon

        MapManager.facilities = self.sitedict['facilities']

        sourcefile = self.sitedict['mainmap']

        res = MapManager._read_map( sourcefile )

        if res == None:

            elog.die( "Shutting down because of failure to read main map file '%s' and metadata" % sourcefile )

        else:

            MapManager.mainmap = res

            MapManager.mainmap['sourcefile'] = sourcefile

        ignore = list()

        for facility_key in MapManager.facilities:

            if 'map' not in MapManager.facilities[facility_key]:

                elog.complain( "Ignoring facility '%s' because parameter-file does not specify a 'map' for it" % facility_key )

                ignore.append( facility_key )

            elif 'lat' not in MapManager.facilities[facility_key]:

                elog.complain( "Ignoring facility '%s' because parameter-file does not specify a 'lat' for it" % facility_key )

                ignore.append( facility_key )

            elif 'lon' not in MapManager.facilities[facility_key]:

                elog.complain( "Ignoring facility '%s' because parameter-file does not specify a 'lon' for it" % facility_key )

                ignore.append( facility_key )

            else:

                sourcefile =  MapManager.facilities[facility_key]['map']

                res = MapManager._read_map( sourcefile )

                if res == None:

                    elog.complain( "Ignoring facility '%s' because of failure to read map file and metadata" % facility_key )
                
                else:
                    
                    MapManager.facilities[facility_key]['image'] = res['image']
                    MapManager.facilities[facility_key]['width'] = res['width']
                    MapManager.facilities[facility_key]['height'] = res['height']
                    MapManager.facilities[facility_key]['proj'] = res['proj']
                    MapManager.facilities[facility_key]['map_latc'] = res['map_latc']
                    MapManager.facilities[facility_key]['map_lonc'] = res['map_lonc']
                    MapManager.facilities[facility_key]['xdelmin'] = res['xdelmin']
                    MapManager.facilities[facility_key]['xdelmax'] = res['xdelmax']
                    MapManager.facilities[facility_key]['ydelmin'] = res['ydelmin']
                    MapManager.facilities[facility_key]['ydelmax'] = res['ydelmax']
                    MapManager.facilities[facility_key]['xscale_pixperdeg'] = res['xscale_pixperdeg']
                    MapManager.facilities[facility_key]['yscale_pixperdeg'] = res['yscale_pixperdeg']
    
                    MapManager.facilities[facility_key]['lat'] = float( MapManager.facilities[facility_key]['lat'] )
                    MapManager.facilities[facility_key]['lon'] = float( MapManager.facilities[facility_key]['lon'] )

                    MapManager.facilities[facility_key]['stacoords'] = dict()

                    for sta in MapManager.facilities[facility_key]['stations']:

                        ( x_deg, y_deg ) = MapManager._edpxy( MapManager.facilities[facility_key]['map_latc'], 
                                                             MapManager.facilities[facility_key]['map_lonc'],
                                                             MapManager.stas[sta]['lat'],
                                                             MapManager.stas[sta]['lon'] )

                        xdelmin = MapManager.facilities[facility_key]['xdelmin']
                        ydelmax = MapManager.facilities[facility_key]['ydelmax']
                        xscale_pixperdeg = MapManager.facilities[facility_key]['xscale_pixperdeg']
                        yscale_pixperdeg = MapManager.facilities[facility_key]['yscale_pixperdeg']
                        symbol_size = int( self.sitedict['symbol_size'] )

                        MapManager.facilities[facility_key]['stacoords'][sta] = dict()

                        MapManager.facilities[facility_key]['stacoords'][sta]['x_px'] = int( ( x_deg - xdelmin ) * xscale_pixperdeg )
                        MapManager.facilities[facility_key]['stacoords'][sta]['y_px'] = int( ( y_deg - ydelmax ) * yscale_pixperdeg )

                        MapManager.facilities[facility_key]['stacoords'][sta]['ulx_px'] = MapManager.facilities[facility_key]['stacoords'][sta]['x_px'] - symbol_size
                        MapManager.facilities[facility_key]['stacoords'][sta]['uly_px'] = MapManager.facilities[facility_key]['stacoords'][sta]['y_px'] - symbol_size
                        MapManager.facilities[facility_key]['stacoords'][sta]['lrx_px'] = MapManager.facilities[facility_key]['stacoords'][sta]['x_px'] + symbol_size
                        MapManager.facilities[facility_key]['stacoords'][sta]['lry_px'] = MapManager.facilities[facility_key]['stacoords'][sta]['y_px'] + symbol_size

        for facility_key in ignore:

            if facility_key in MapManager.facilities:

                del MapManager.facilities[facility_key]

        MapManager.mainmap['faccoords'] = dict()

        for facility_key in MapManager.facilities:

            ( x_deg, y_deg ) = MapManager._edpxy( MapManager.mainmap['map_latc'], 
                                                     MapManager.mainmap['map_lonc'],
                                                     MapManager.facilities[facility_key]['lat'],
                                                     MapManager.facilities[facility_key]['lon'] )

            xdelmin = MapManager.mainmap['xdelmin']
            ydelmax = MapManager.mainmap['ydelmax']
            xscale_pixperdeg = MapManager.mainmap['xscale_pixperdeg']
            yscale_pixperdeg = MapManager.mainmap['yscale_pixperdeg']
            symbol_size = int( self.sitedict['symbol_size'] )

            MapManager.mainmap['faccoords'][facility_key] = dict()

            MapManager.mainmap['faccoords'][facility_key]['x_px'] = int( ( x_deg - xdelmin ) * xscale_pixperdeg )
            MapManager.mainmap['faccoords'][facility_key]['y_px'] = int( ( y_deg - ydelmax ) * yscale_pixperdeg )

            MapManager.mainmap['faccoords'][facility_key]['ulx_px'] = MapManager.mainmap['faccoords'][facility_key]['x_px'] - symbol_size
            MapManager.mainmap['faccoords'][facility_key]['uly_px'] = MapManager.mainmap['faccoords'][facility_key]['y_px'] - symbol_size
            MapManager.mainmap['faccoords'][facility_key]['lrx_px'] = MapManager.mainmap['faccoords'][facility_key]['x_px'] + symbol_size
            MapManager.mainmap['faccoords'][facility_key]['lry_px'] = MapManager.mainmap['faccoords'][facility_key]['y_px'] + symbol_size

    @staticmethod
    def _read_map( sourcefile ):

        if not os.path.exists( sourcefile + ".pf" ):

            elog.complain( "Could not find map pf file '%s'" % ( sourcefile + ".pf" ) )

            return None

        res = dict()

        res['image'] = Image.open( sourcefile )

        size = res['image'].size

        res['width'] = size[0]
        res['height'] = size[1]

        mappfptr = stock.pfin( sourcefile + ".pf" )
                
        if mappfptr == None:

            elog.complain( "Failed to open map pf file '%s'" % ( sourcefile + ".pf" ) )

            return None

        mappfdict = mappfptr.pf2dict()

        if mappfdict == None or type( mappfdict ) != dict or len( mappfdict ) != 1:

            elog.complain( "Map pf file '%s' must have one and only one sub-array" % ( sourcefile + ".pf" ) )

            return None

        akey = list(mappfdict.keys())[0]

        if 'proj' in mappfdict[akey]:

            res['proj'] = mappfdict[akey]['proj']

        else: 

            elog.complain( "Map pf file '%s' is missing 'proj' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'latc' in mappfdict[akey]:

            res['map_latc'] = float( mappfdict[akey]['latc'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'latc' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'lonc' in mappfdict[akey]:

            res['map_lonc'] = float( mappfdict[akey]['lonc'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'lonc' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'xdelmin' in mappfdict[akey]:

            res['xdelmin'] = float( mappfdict[akey]['xdelmin'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'xdelmin' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'xdelmax' in mappfdict[akey]:

            res['xdelmax'] = float( mappfdict[akey]['xdelmax'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'xdelmax' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'ydelmin' in mappfdict[akey]:

            res['ydelmin'] = float( mappfdict[akey]['ydelmin'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'ydelmin' parameter" % ( sourcefile + ".pf" ) )

            return None

        if 'ydelmax' in mappfdict[akey]:

            res['ydelmax'] = float( mappfdict[akey]['ydelmax'] )

        else: 

            elog.complain( "Map pf file '%s' is missing 'ydelmax' parameter" % ( sourcefile + ".pf" ) )

            return None

        res['xscale_pixperdeg'] = res['width'] / ( res['xdelmax'] - res['xdelmin'] )
        res['yscale_pixperdeg'] = res['height'] / ( res['ydelmin'] - res['ydelmax'] )

        return res


    @staticmethod
    def _edpxy( latc, lonc, lat, lon ):

        if latc == lat and lonc == lon:

            x_deg = 0.0
            y_deg = 0.0
        
        else:

            ( distance, azimuth ) = coords.dist( latc, lonc, lat, lon )

            azimuth = coords.rad( azimuth )

            x_deg = distance * math.sin( azimuth )
            y_deg = distance * math.cos( azimuth )

        return ( x_deg, y_deg )

    def decorated_imageblock( self, facility_key, statusmanager ):

        if facility_key == 'overall':

            im =  MapManager.mainmap['image']

            im = im.convert( 'RGB' )

            draw = ImageDraw.Draw( im )

            for facility_key in MapManager.facilities:

                ( status, color, statusmsg ) = statusmanager.status_facility( facility_key )

                draw.rectangle( ( MapManager.mainmap['faccoords'][facility_key]['ulx_px'], 
                                  MapManager.mainmap['faccoords'][facility_key]['uly_px'], 
                                  MapManager.mainmap['faccoords'][facility_key]['lrx_px'], 
                                  MapManager.mainmap['faccoords'][facility_key]['lry_px'] ), 
                                  fill=color, outline="black" )

            del draw

        else:

            im =  MapManager.facilities[facility_key]['image']

            im = im.convert( 'RGB' )

            draw = ImageDraw.Draw( im )

            for sta in MapManager.facilities[facility_key]['stations']:

                ( status, color, statusmsg ) = statusmanager.status_station( sta )

                draw.rectangle( ( MapManager.facilities[facility_key]['stacoords'][sta]['ulx_px'], 
                                  MapManager.facilities[facility_key]['stacoords'][sta]['uly_px'], 
                                  MapManager.facilities[facility_key]['stacoords'][sta]['lrx_px'], 
                                  MapManager.facilities[facility_key]['stacoords'][sta]['lry_px'] ), 
                                  fill=color, outline="black" )

            del draw

        tempfile = '/tmp/mapwithsymbols_%s_%d_%d.somefmt' % ( facility_key, os.getuid(), os.getpid() )

        im.save( tempfile, format="GIF" )

        del im

        f = open( tempfile, 'r' )

        tblock = f.read()

        f.close()

        os.unlink( tempfile ) 

        return tblock

    def imagemap( self, facility_key, width ): 

        imagemap = """
                   <map name="symbolmap">
                   """

        if facility_key == 'overall':

            # Assume x and y scaling are both defined by the target end width:

            scale = float( width ) / float( MapManager.mainmap['width'] )

            for facility_key in MapManager.facilities:

                ulx_scaled = int( MapManager.mainmap['faccoords'][facility_key]['ulx_px'] * scale )
                uly_scaled = int( MapManager.mainmap['faccoords'][facility_key]['uly_px'] * scale )
                lrx_scaled = int( MapManager.mainmap['faccoords'][facility_key]['lrx_px'] * scale )
                lry_scaled = int( MapManager.mainmap['faccoords'][facility_key]['lry_px'] * scale )

                imagemap += """
                              <area shape="rect" coords="%d,%d,%d,%d" href="/facility/%s" alt="%s facility" title="Facility: %s" />
                                   """ % ( ulx_scaled, uly_scaled, lrx_scaled, lry_scaled, facility_key, facility_key, facility_key )

        else:

            # Assume x and y scaling are both defined by the target end width:

            scale = float( width ) / float( MapManager.facilities[facility_key]['width'] )

            for sta in MapManager.facilities[facility_key]['stations']:

                ulx_scaled = int( MapManager.facilities[facility_key]['stacoords'][sta]['ulx_px'] * scale )
                uly_scaled = int( MapManager.facilities[facility_key]['stacoords'][sta]['uly_px'] * scale )
                lrx_scaled = int( MapManager.facilities[facility_key]['stacoords'][sta]['lrx_px'] * scale )
                lry_scaled = int( MapManager.facilities[facility_key]['stacoords'][sta]['lry_px'] * scale )

                imagemap += """
                              <area shape="rect" coords="%d,%d,%d,%d" href="/stationalarms/%s" alt="Station: %s" title="Station: %s" />
                                   """ % ( ulx_scaled, uly_scaled, lrx_scaled, lry_scaled, sta, sta, sta )

        imagemap += """
                          </map>
                           """ 

        return imagemap
