
import os
import re
import math

import antelope.orb as orb
import antelope.Pkt as Pkt
import antelope.elog as elog
import antelope.stock as stock
import antelope.coords as coords
import antelope.datascope as datascope

from twisted.web import http


class DemoWeb(object):

    stdct = None

    def __init__(self, pfdict):

        if DemoWeb.stdct is None:
            DemoWeb.stdct = pfdict

    @property
    def centerimage_width(self):
        return DemoWeb.stdct['siteconfig']['centerimage_width']

    @property
    def refresh_sec(self):
        return int(DemoWeb.stdct['siteconfig']['refresh_sec'])

    def write_htmlhead(self, request, title, refresh=True):
        """ Populate request with html <title> element

        :param request:
        :param title:
        :param refresh:
        :return: None
        """

        request.write(DemoWeb.stdct['phrases']['htmlhead_open'].encode('utf-8'))

        if refresh and self.refresh_sec != 0:
            request.write(('<meta http-equiv="refresh" content="%d">' % self.refresh_sec).encode('utf-8'))

        request.write(("<title>%s</title>" % title).encode('utf-8'))

    def write_htmlbodystart(self, request):
        """

        :param request:
        :return:
        """

        request.write("<body>".encode('utf-8'))

        request.write("""<div id="pagecontainer">""".encode('utf-8'))

        request.write(DemoWeb.stdct['phrases']['header'].encode('utf-8'))

        request.write("""
                <div id="content">
                """.encode('utf-8'))

    def write_htmlbodyfinish(self, request, refresh=True):
        """

        :param request:
        :param refresh:
        :return: None
        """
        if refresh and self.refresh_sec != 0:

            request.write(("<h3><-  Page updated %s (refreshes every %d seconds)  -></h3>\n" %
                          (stock.epoch2str( stock.now(), DemoWeb.stdct['siteconfig']['time_format'], ""),
                           self.refresh_sec)).encode('utf-8'))

        request.write("</div> <!-- content -->".encode('utf-8'))

        request.write(DemoWeb.stdct['phrases']['footer'].encode('utf-8'))

        request.write("""
                </div>        <!-- pagecontainer -->
                </body>
                </html>
                """.encode('utf-8'))

        request.finish()

    def write_mainnav(self, request, activepage=None):
        """ Populate with relevant html data based on current active view

        :param request:
        :param activepage:
        :return: None
        """
        request.write("""
                    <div id="nav">
                  <ul id="main_menu">
                """.encode('utf-8'))

        if activepage == 'dbevents':
            request.write("""
                  <li><a class="activepage" href="/">Dbevents</a></li>
                """.encode('utf-8'))
        else:
            request.write("""
                  <li><a href="/">Dbevents</a></li>
                """.encode('utf-8'))

        if activepage == 'webrtd':
            request.write("""
                  <li><a class="activepage" href="/webrtd">rtd</a></li>
               """.encode('utf-8'))
        else:
            request.write("""
                  <li><a href="/webrtd">rtd</a></li>
               """.encode('utf-8'))

        if activepage == 'sources':
            request.write("""
                  <li><a class="activepage" href="/sources">sources</a></li>
               """.encode('utf-8'))
        else:
            request.write("""
                  <li><a href="/sources">sources</a></li>
               """.encode('utf-8'))

        if activepage == 'clients':
            request.write("""
                  <li><a class="activepage" href="/clients">clients</a></li>
               """.encode('utf-8'))
        else:
            request.write("""
                  <li><a href="/clients">clients</a></li>
               """.encode('utf-8'))

        if activepage == 'rtm':
            request.write("""
                  <li><a class="activepage" href="/rtm">RTM</a></li>
               """.encode('utf-8'))
        else:
            request.write("""
                  <li><a href="/rtm">RTM</a></li>
               """.encode('utf-8'))

        request.write("""
                </ul>
                </div>
                """.encode('utf-8'))

    def write_notfound(self, request):
        """ Set response for http.NOT_FOUND, and add corresponding header element
        :param request:
        :return: None
        """

        request.setResponseCode(http.NOT_FOUND)

        request.write("Requested page not found".encode('utf-8'))

        request.finish()

    def write_forbidden(self, request):
        """ Set response for http.FORBIDDEN, and add corresponding header element
        :param request:
        :return: None
        """

        request.setResponseCode(http.FORBIDDEN)

        request.write("<h1>Access Forbidden</h1>".encode('utf-8'))

        request.finish()

